@@ -81,8 +81,11 @@ DocfileManagerWidget::DocfileManagerWidget(QWidget* parent)
     QFrame* separator2 = new QFrame();
     separator2->setFrameShape(QFrame::HLine);
     QPushButton* openFileManagerButton = new QPushButton(i18n("Open File Manager"));
+    openFileManagerButton->setIcon(QIcon::fromTheme(QStringLiteral("system-file-manager")));
     QPushButton* openTextEditorButton = new QPushButton(i18nc("Edit selected files", "Edit selected"));
+    openTextEditorButton->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
     QPushButton* searchPathsButton = new QPushButton(i18n("Search Paths..."));
+    searchPathsButton->setIcon(QIcon::fromTheme(QStringLiteral("search")));
     buttonsLayout->addWidget(separator);
     buttonsLayout->addWidget(openFileManagerButton);
     buttonsLayout->addWidget(openTextEditorButton);
