@@ -24,12 +24,9 @@
 #include <serialization/indexedstring.h>
 
 #include "pythonduchainexport.h"
-#include "decorator.h"
 
 namespace Python {
 
-KDEVPYTHONDUCHAIN_EXPORT DECLARE_LIST_MEMBER_HASH(FunctionDeclarationData, m_decorators, Decorator);
-
 class KDEVPYTHONDUCHAIN_EXPORT FunctionDeclarationData : public KDevelop::FunctionDeclarationData
 {
 public:
@@ -37,35 +34,25 @@ public:
         : KDevelop::FunctionDeclarationData()
         , m_isStatic(false)
         , m_isClassMethod(false)
+        , m_isProperty(false)
         , m_vararg(-1)
-        , m_kwarg(-1)
-    {
-        initializeAppendedLists();
-    }
+        , m_kwarg(-1) {}
 
     FunctionDeclarationData(const FunctionDeclarationData& rhs)
         : KDevelop::FunctionDeclarationData(rhs)
         , m_isStatic(rhs.m_isStatic)
         , m_isClassMethod(rhs.m_isClassMethod)
+        , m_isProperty(rhs.m_isProperty)
         , m_vararg(rhs.m_vararg)
-        , m_kwarg(rhs.m_kwarg)
-    {
-        initializeAppendedLists();
-        copyListsFrom(rhs);
-    }
-    
-    ~FunctionDeclarationData() {
-        freeAppendedLists();
-    }
-    
+        , m_kwarg(rhs.m_kwarg) {}
+
+    ~FunctionDeclarationData() {}
+
     bool m_isStatic: 1;
     bool m_isClassMethod: 1;
+    bool m_isProperty: 1; //TODO real property declarations
     short m_vararg;
     short m_kwarg;
-
-    START_APPENDED_LISTS_BASE(FunctionDeclarationData, KDevelop::FunctionDeclarationData);
-    APPENDED_LIST_FIRST(FunctionDeclarationData, Decorator, m_decorators);
-    END_APPENDED_LISTS(FunctionDeclarationData, m_decorators);
 };
 
 class KDEVPYTHONDUCHAIN_EXPORT FunctionDeclaration : public KDevelop::FunctionDeclaration
@@ -111,26 +98,22 @@ public:
     inline void setClassMethod(bool isClassMethod) {
         d_func_dynamic()->m_isClassMethod = isClassMethod;
     }
-    
-    inline const Decorator* decorators() const {
-        return d_func()->m_decorators();
-    };
-    
-    inline unsigned int decoratorsSize() const {
-        return d_func()->m_decoratorsSize();
-    };
-    
-    inline void addDecorator(const Decorator& d) {
-        d_func_dynamic()->m_decoratorsList().insert(0, d);
+
+    inline bool isProperty() const {
+        return d_func()->m_isProperty;
     }
-    
-    typedef DUChainPointer<FunctionDeclaration> Ptr;
+
+    inline void setProperty(bool isProperty) {
+        d_func_dynamic()->m_isProperty = isProperty;
+    }
+
+    typedef KDevelop::DUChainPointer<FunctionDeclaration> Ptr;
     
 private:
     DUCHAIN_DECLARE_DATA(FunctionDeclaration);
 };
 
-typedef DUChainPointer<FunctionDeclaration> FunctionDeclarationPointer;
+typedef KDevelop::DUChainPointer<FunctionDeclaration> FunctionDeclarationPointer;
 } // namespace Python
 
 #endif // PYTHONFUNCTIONDECLARATION_H
