@@ -57,6 +57,8 @@
 
 #include "duchain/helpers.h"
 
+#include "kdevpythonversion.h"
+
 QTEST_MAIN(PyDUChainTest)
 
 using namespace KDevelop;
@@ -425,6 +427,20 @@ void PyDUChainTest::testCrashes_data() {
         "    return MyClass()\n"
         "foo([0]).bar()";
     QTest::newRow("unpacked_dict_kwarg") << "def foo(arg): pass\nfoo(**{'arg': 2})";
+    QTest::newRow("negative_container_hints") <<
+        "class Evil:\n"
+        "   def aa(self, arg):\n"
+        "      \"\"\"! addsTypeOfArgContent ! -1\"\"\"\n"
+        "   def bb(self, arg):\n"
+        "      \"\"\"! addsTypeOfArg ! -2\"\"\"\n"
+        "   def cc(self, arg):\n"
+        "      \"\"\"! returnContentEqualsContentOf ! -3\"\"\"\n"
+        "e = Evil()\n"
+        "z = [e.aa(1), e.bb(2), e.cc(3)]";
+#if PYTHON_VERSION >= QT_VERSION_CHECK(3, 6, 0)
+    QTest::newRow("comprehension_in_fstring") <<
+        "def crash(): return f'expr={ {x: y for x, y in [(1, 2), ]}}'";
+#endif
 }
 
 void PyDUChainTest::testClassVariables()
@@ -790,19 +806,25 @@ void PyDUChainTest::testTypes()
     
     QFETCH(QString, code);
     QFETCH(QString, expectedType);
-    
+
     ReferencedTopDUContext ctx = parse(code.toUtf8());
     QVERIFY(ctx);
     QVERIFY(m_ast);
-    
+
     DUChainReadLocker lock(DUChain::lock());
     TypeTestVisitor* visitor = new TypeTestVisitor();
     visitor->ctx = TopDUContextPointer(ctx.data());
     visitor->searchingForType = expectedType;
     visitor->visitCode(m_ast.data());
+    QEXPECT_FAIL("tuple_func", "no suitable docstring hint", Continue);
+    QEXPECT_FAIL("tuple_slice", "not implemented", Continue);
+    QEXPECT_FAIL("tuple_add", "not implemented", Continue);
+    QEXPECT_FAIL("tuple_mul", "not implemented", Continue);
     QEXPECT_FAIL("return_builtin_iterator", "fake builtin iter()", Continue);
     QEXPECT_FAIL("parent_constructor_arg_type", "Not enough passes?", Continue);
     QEXPECT_FAIL("init_class_no_decl", "aliasing info lost", Continue);
+    QEXPECT_FAIL("property_wrong", "visitCall uses declaration if no type", Continue);
+    QEXPECT_FAIL("property_setter", "very basic property support", Continue);
     QCOMPARE(visitor->found, true);
 }
 
@@ -810,7 +832,11 @@ void PyDUChainTest::testTypes_data()
 {
     QTest::addColumn<QString>("code");
     QTest::addColumn<QString>("expectedType");
-    
+
+#if PYTHON_VERSION >= QT_VERSION_CHECK(3, 6, 0)
+    QTest::newRow("annotate_decl") << "checkme: int" << "int";
+    QTest::newRow("annotate_assign") << "checkme: int = 3.5" << "unsure (float, int)";
+#endif
     QTest::newRow("listtype") << "checkme = []" << "list";
     QTest::newRow("listtype_func") << "checkme = list()" << "list";
     QTest::newRow("listtype_with_contents") << "checkme = [1, 2, 3, 4, 5]" << "list of int";
@@ -819,6 +845,11 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("dicttype_get") << "d = {0.4:5}; checkme = d.get(0)" << "int";
     QTest::newRow("dicttype_func") << "checkme = dict()" << "dict";
     QTest::newRow("dicttype_extended") << "some_misc_var = {}; checkme = some_misc_var" << "dict";
+    QTest::newRow("tuple") << "checkme = ()" << "tuple of ()";
+    QTest::newRow("tuple_func") << "checkme = tuple((1, 2.3))" << "tuple of (int, float)";
+    QTest::newRow("tuple_with_contents") << "checkme = 1, 2.3" << "tuple of (int, float)";
+    QTest::newRow("tuple_extended") << "some_misc_var = (); checkme = some_misc_var" << "tuple of ()";
+    QTest::newRow("tuple_max_display") << "checkme = 1,2,3,4,5,6" << "tuple of (int, int, int, int, int, ...)";
     QTest::newRow("bool") << "checkme = True" << "bool";
     QTest::newRow("float") << "checkme = 3.7" << "float";
     QTest::newRow("int") << "checkme = 3" << "int";
@@ -845,18 +876,21 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("list_access_closed_slice") << "some_list = []; checkme = some_list[2:17]" << "list";
     QTest::newRow("list_access_step") << "some_list = []; checkme = some_list[::2]" << "list";
     QTest::newRow("list_access_singleItem") << "some_list = []; checkme = some_list[42]" << "mixed";
-    
+
     QTest::newRow("funccall_number") << "def foo(): return 3; \ncheckme = foo();" << "int";
     QTest::newRow("funccall_string") << "def foo(): return 'a'; \ncheckme = foo();" << "str";
     QTest::newRow("funccall_list") << "def foo(): return []; \ncheckme = foo();" << "list";
     QTest::newRow("funccall_dict") << "def foo(): return {}; \ncheckme = foo();" << "dict";
-    
+    QTest::newRow("funccall_no_return") << "def foo(): pass\ncheckme = foo()" << "None";
+    QTest::newRow("funccall_def_return") << "def foo(): return\ncheckme = foo()" << "None";
+    QTest::newRow("funccall_maybe_def_return") << "def foo():\n if False: return\n return 7\ncheckme = foo()" << "unsure (None, int)";
+
     QTest::newRow("tuple1") << "checkme, foo = 3, \"str\"" << "int";
     QTest::newRow("tuple2") << "foo, checkme = 3, \"str\"" << "str";
     QTest::newRow("tuple2_negative_index") << "foo = (1, 2, 'foo')\ncheckme = foo[-1]" << "str";
     QTest::newRow("tuple_type") << "checkme = 1, 2" << "tuple";
     QTest::newRow("tuple_rhs_unpack") << "foo = 1, 2.5\nbar = 1, *foo, 2\ncheckme = bar[2]" << "float";
-    
+
     QTest::newRow("dict_iteritems") << "d = {1:2, 3:4}\nfor checkme, k in d.iteritems(): pass" << "int";
     QTest::newRow("enumerate_key") << "d = [str(), str()]\nfor checkme, value in enumerate(d): pass" << "int";
     QTest::newRow("enumerate_value") << "d = [str(), str()]\nfor key, checkme in enumerate(d): pass" << "str";
@@ -864,9 +898,14 @@ void PyDUChainTest::testTypes_data()
 
     QTest::newRow("dict_assign_twice") << "d = dict(); d[''] = 0; d = dict(); d[''] = 0; checkme = d"
                                        << "unsure (dict of str : int, dict)";
-    
+
     QTest::newRow("class_method_import") << "class c:\n attr = \"foo\"\n def m():\n  return attr;\n  return 3;\ni=c()\ncheckme=i.m()" << "int";
-    QTest::newRow("getsListDecorator") << "foo = [1, 2, 3]\ncheckme = foo.reverse()" << "list of int";
+    QTest::newRow("getsListDocstring") << "foo = [1, 2, 3]\ncheckme = foo.reverse()" << "list of int";
+
+    QTest::newRow("str_iter") << "checkme = [char for char in 'Hello, world!']" << "list of str";
+    QTest::newRow("str_subscript") << "checkme = 'Hello, world!'[0]" << "str";
+    QTest::newRow("bytes_iter") << "checkme = [byte for byte in b'Hello, world!']" << "list of int";
+    QTest::newRow("bytes_subscript") << "checkme = b'Hello, world!'[0]" << "int";
 
     QTest::newRow("fromAssertIsinstance") << "class c(): pass\ncheckme = mixed()\nassert isinstance(checkme, c)\n" << "c";
     QTest::newRow("fromAssertIsinstanceInvalid") << "class c(): pass\ncheckme = mixed()\nassert isinstance(c, checkme)\n" << "mixed";
@@ -932,6 +971,16 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("tuple_indexaccess") << "t = 3, 5.5\ncheckme = t[0]" << "int";
     QTest::newRow("tuple_indexaccess2") << "t = 3, 5.5\ncheckme = t[1]" << "float";
     QTest::newRow("tuple_indexaccess3") << "t = 3, 4\ncheckme = t[1]" << "int";
+    QTest::newRow("tuple_indexaccess4") << "t = 3, 4.5\ncheckme = t[2]" << "unsure (int, float)";
+
+    QTest::newRow("tuple_indexaccess_neg") << "t = 3, 4.5; checkme = t[-1]" << "float";
+    QTest::newRow("tuple_indexaccess_neg2") << "t = 3, 4.5; checkme = t[-2]" << "int";
+    QTest::newRow("tuple_indexaccess_neg3") << "t = 3, 4.5; checkme = t[-3]" << "unsure (int, float)";
+
+    QTest::newRow("tuple_slice") << "t = 3, 'q', 4.5; checkme = t[-3: 2]" << "tuple of (int, str)";
+
+    QTest::newRow("tuple_add") << "t, u = (3,), ('q', 4.5); checkme = t + u" << "tuple of (int, str, float)";
+    QTest::newRow("tuple_mul") << "t = 3, 4.5; checkme = t * 2" << "tuple of (int, float, int, float)";
 
     QTest::newRow("dict_unsure") << "t = dict(); t = {3: str()}\ncheckme = t[1].capitalize()" << "str";
     QTest::newRow("unsure_attr_access") << "d = str(); d = 3; checkme = d.capitalize()" << "str";
@@ -945,10 +994,12 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("args_type") << "def myfun(*args): return args[0]\ncheckme = myfun(3)" << "int";
     QTest::newRow("kwarg_type") << "def myfun(**kwargs): return kwargs['a']\ncheckme = myfun(a=3)" << "int";
     QTest::newRow("dict_kwarg_type") << "def foo(**kwargs): return kwargs['']\ncheckme = foo(**{'a': 12})" << "int";
+#if PYTHON_VERSION >= QT_VERSION_CHECK(3, 5, 0)
     QTest::newRow("dict_norm_kwarg_type") << "def foo(**kwargs): return kwargs['']\n"
                                              "checkme = foo(**{'a': 12}, b=1.2)" << "unsure (int, float)";
     QTest::newRow("multi_dict_kwarg_type") << "def foo(**kwargs): return kwargs['']\n"
                                               "checkme = foo(**{'a': 12}, b=1.2, **{'c': ''})" << "unsure (int, float, str)";
+#endif
     QTest::newRow("named_arg_type") << "def myfun(arg): return arg\ncheckme = myfun(arg=3)" << "int";
 
     QTest::newRow("arg_args_type") << "def myfun(arg, *args): return args[0]\n"
@@ -967,6 +1018,10 @@ void PyDUChainTest::testTypes_data()
                                            "checkme = myfun(1, 1.5, a=str())" << "float";
     QTest::newRow("varied_args_type_3") << "def myfun(arg, *args, **kwargs): return kwargs['a']\n"
                                            "checkme = myfun(1, 1.5, a=str())" << "str";
+    QTest::newRow("nested_arg_name_type") << "def foo(xxx):\n"
+                                             "  def bar(xxx): return xxx\n"
+                                             "  return bar('test')\n"
+                                             "checkme = foo(10)\n" << "str";
     QTest::newRow("method_args_type_1") << "class MyClass:\n"
                                            "   def method(self, arg): return self\n"
                                            "checkme = MyClass().method(12)" << "MyClass";
@@ -1015,6 +1070,16 @@ void PyDUChainTest::testTypes_data()
                                                     "       Base.__init__(self, foo)\n"
                                                     "instance = Derived('string')\n"
                                                     "checkme = instance.foo" << "str";
+    QTest::newRow("nested_class_self_inside") << "class Foo:\n"
+                                                 "   def foo(self):\n"
+                                                 "       class Bar:\n"
+                                                 "           def bar(self): return self\n"
+                                                 "       return Bar().bar()\n"
+                                                 "checkme = Foo().foo()\n" << "Foo::foo::Bar";
+    QTest::newRow("nested_class_self_after") << "class Foo:\n"
+                                                "    class Bar: pass\n"
+                                                "    def foo(self): return self\n"
+                                                "checkme = Foo().foo()\n" << "Foo";
 
     QTest::newRow("tuple_unsure") << "q = (3, str())\nq=(str(), 3)\ncheckme, _ = q" << "unsure (int, str)";
 
@@ -1058,11 +1123,25 @@ void PyDUChainTest::testTypes_data()
                                     "         return k\n"
                                     "f = Foo.foo()\n"
                                     "checkme = f\n" << "Foo";
+    QTest::newRow("property_getter") << "class Foo:\n"
+                                        "    @property\n"
+                                        "    def bar(self): return 35\n"
+                                        "checkme = Foo().bar" << "int";
+    QTest::newRow("property_wrong") << "class Foo:\n"
+                                       "    @property\n"
+                                       "    def bar(self): return True\n"
+                                       "checkme = Foo().bar()" << "mixed";
+    QTest::newRow("property_setter") << "class Foo:\n"
+                                        "    @property\n"
+                                        "    def bar(self): return 35\n"
+                                        "    @bar.setter\n"
+                                        "    def bar(self, value): return 18.3\n" // Return should be ignored
+                                        "checkme = Foo().bar" << "int";
 
     QTest::newRow("tuple_listof") << "l = [(1, 2), (3, 4)]\ncheckme = l[1][0]" << "int";
 
     QTest::newRow("getitem") << "class c:\n def __getitem__(self, slice): return 3.14\na = c()\ncheckme = a[2]" << "float";
-    
+
     QTest::newRow("constructor_type_deduction") << "class myclass:\n"
                                                    "\tdef __init__(self, param): self.foo=param\n"
                                                    "checkme = myclass(3).foo" << "int";
@@ -1179,6 +1258,10 @@ void PyDUChainTest::testProblemCount_data()
                                         "  [x for a in [1, 2, 3]]" << 1;
     QTest::newRow("list_comp_staticmethod_wrong") << "class A:\n @staticmethod\n def func(cls):\n"
                                         "  [x for a in [1, 2, 3]]" << 1;
+    QTest::newRow("misplaced_return_plain") << "return" << 1;
+    QTest::newRow("misplaced_return_value") << "return 15" << 1;
+    QTest::newRow("misplaced_return_class") << "class A:\n return 25" << 1;
+    QTest::newRow("correct_return") << "def foo():\n return" << 0;
 }
 
 void PyDUChainTest::testImportDeclarations_data() {
@@ -1279,7 +1362,7 @@ void PyDUChainTest::testFunctionHints_data()
     QTest::addColumn<QString>("code");
     QTest::addColumn<QString>("expectedType");
     
-    QTest::newRow("func_return_type") << "def myfun(arg) -> int: pass\ncheckme = myfun(\"3\")" << "unsure (void, int)";
+    QTest::newRow("func_return_type") << "def myfun(arg) -> int: pass\ncheckme = myfun(\"3\")" << "unsure (None, int)";
     QTest::newRow("argument_type") << "def myfun(arg : int): return arg\ncheckme = myfun(foobar)" << "int";
     QTest::newRow("argument_type_only_if_typeof") << "def myfun(arg : 3): return arg\ncheckme = myfun(foobar)" << "mixed";
 }
@@ -1308,33 +1391,6 @@ void PyDUChainTest::testHintedTypes_data()
     QTest::newRow("unsure_attribute") << "def myfunc(x): return x.capitalize()\nmyfunc(3.5)\ncheckme = myfunc(str())" << "str";
 }
 
-void PyDUChainTest::testDecorators()
-{
-    QFETCH(QString, code);
-//     QFETCH(int, amountOfDecorators);
-    QFETCH(QStringList, names);
-    ReferencedTopDUContext ctx = parse(code);
-    QVERIFY(ctx);
-    DUChainReadLocker lock(DUChain::lock());
-    Python::FunctionDeclaration* decl = dynamic_cast<Python::FunctionDeclaration*>(
-        ctx->allDeclarations(CursorInRevision::invalid(), ctx->topContext()).first().first);
-    QVERIFY(decl);
-    foreach ( const QString& decoratorName, names ) {
-        QVERIFY(Helper::findDecoratorByName<Python::FunctionDeclaration>(decl, decoratorName));
-    }
-}
-
-void PyDUChainTest::testDecorators_data()
-{
-    QTest::addColumn<QString>("code");
-    QTest::addColumn<int>("amountOfDecorators");
-    QTest::addColumn<QStringList>("names");
-    
-    QTest::newRow("one_decorator") << "@foo\ndef func(): pass" << 1 << ( QStringList() << "foo" );
-    QTest::newRow("decorator_with_args") << "@foo(2, \"bar\")\ndef func(): pass" << 1 << ( QStringList() << "foo");
-    QTest::newRow("two_decorators") << "@foo\n@bar(17)\ndef func(): pass" << 2 << ( QStringList() << "foo" << "bar" );
-}
-
 void PyDUChainTest::testOperators()
 {
     QFETCH(QString, code);
@@ -1409,7 +1465,7 @@ void PyDUChainTest::testInheritance()
     bool classDeclFound = false;
     foreach ( const p& item, decls ) {
         if ( item.first->identifier().toString() == "B" ) {
-            auto klass = dynamic_cast<Python::ClassDeclaration*>(item.first);
+            auto klass = dynamic_cast<ClassDeclaration*>(item.first);
             QVERIFY(klass);
             QCOMPARE(klass->baseClassesSize(), static_cast<unsigned int>(expectedBaseClasses));
             classDeclFound = true;
@@ -1481,16 +1537,19 @@ void PyDUChainTest::testContainerTypes_data()
     QTest::newRow("generator") << "checkme = [i for i in [1, 2, 3]]" << "int" << false;
     QTest::newRow("list_access") << "list = [1, 2, 3]\ncheckme = list[0]" << "int" << true;
     QTest::newRow("set_of_int") << "checkme = {1, 2, 3}" << "int" << false;
-    QTest::newRow("set_from_unpacked") << "foo = [1.3]\ncheckme = {1, *foo, 3}" << "unsure (int, float)" << false;
     QTest::newRow("set_of_int_call") << "checkme = set({1, 2, 3})" << "int" << false;
     QTest::newRow("set_generator") << "checkme = {i for i in [1, 2, 3]}" << "int" << false;
+    QTest::newRow("frozenset_of_int_call") << "checkme = frozenset({1, 2, 3})" << "int" << false;
     QTest::newRow("dict_of_int") << "checkme = {a:1, b:2, c:3}" << "int" << false;
-    QTest::newRow("dict_from_unpacked") << "checkme = {**{'a': 1}}" << "dict of str : int" << true;
-    QTest::newRow("dict_from_varied") << "checkme = {**{'a': 1}, 1: 1.5}" <<
-                                         "dict of unsure (str, int) : unsure (int, float)" << true;
     QTest::newRow("dict_of_int_call") << "checkme = dict({a:1, b:2, c:3})" << "int" << false;
     QTest::newRow("dict_generator") << "checkme = {\"Foo\":i for i in [1, 2, 3]}" << "int" << false;
     QTest::newRow("dict_access") << "list = {a:1, b:2, c:3}\ncheckme = list[0]" << "int" << true;
+#if PYTHON_VERSION >= QT_VERSION_CHECK(3, 5, 0)
+    QTest::newRow("set_from_unpacked") << "foo = [1.3]\ncheckme = {1, *foo, 3}" << "unsure (int, float)" << false;
+    QTest::newRow("dict_from_unpacked") << "checkme = {**{'a': 1}}" << "dict of str : int" << true;
+    QTest::newRow("dict_from_varied") << "checkme = {**{'a': 1}, 1: 1.5}" <<
+                                         "dict of unsure (str, int) : unsure (int, float)" << true;
+#endif
     QTest::newRow("generator_attribute") << "checkme = [item.capitalize() for item in ['foobar']]" << "str" << false;
     QTest::newRow("cannot_change_type") << "checkme = [\"Foo\", \"Bar\"]" << "str" << false;
     QTest::newRow("cannot_change_type2") << "[1, 2, 3].append(5)\ncheckme = [\"Foo\", \"Bar\"]" << "str" << false;
